# If PTYPE=0 then it will auto-detect the partition type - else set PTYPE as required
set PTYPE=0

# #### DO NOT CHANGE ANY LINES BELOW HERE #####

function test_lba {
# set USED to 0 if LBA is OK to use as a backup sector
set LBA=${1}
set USED=1
hexdump -q -s 0 -n 1 (${grubfm_disk})$LBA+1 b0
set b16=${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}${b0}
set b80=${b16}${b16}${b16}${b16}${b16}${b16}${b16}${b16}
hexdump -q -s 0 -n 0x80 (${grubfm_disk})$LBA+1 b1
if [ ${b80} = $b1 ] ; then
	hexdump -q -s 0x80 -n 0x80 (${grubfm_disk})$LBA+1 b1
	if [ "${b80}" = "$b1" ] ; then
		hexdump -q -s 0x100 -n 0x80 (${grubfm_disk})$LBA+1 b1
		if [ "${b80}" = "$b1" ] ; then
			hexdump -q -s 0x180 -n 0x80 (${grubfm_disk})$LBA+1 b1
			if [ "${b80}" = "$b1" ] ; then
			set USED=0
			echo LBA$LBA contains all $b0
			fi
		fi
	fi
fi
	
# if LBA already contains an MBR then it is OK to use
hexdump -q -s 0x1fe -n 2 (${grubfm_disk})$LBA+1 x
if [ "$x" = "\x55\xaa" ] ; then 
	hexdump -q -s 0x1c2 -n 1 (${grubfm_disk})$LBA+1 x
	if [ "$x" = "\x07" ] ; then 
		hexdump -q -s 0x1d2 -n 1 (${grubfm_disk})$LBA+1 x
		if [ "$x" = "\x0C" -o "$x" = "\x0B" ] ; then 
			set USED=0
			echo LBA$LBA is a valid MBR
		fi
	fi
fi
unset b1
unset b16
unset b80
unset x
unset LBA
}


function check_mbr {
# set USED to 0 if LBA is valid MBR with partition table
set LBA=${1}
set USED=1
# if LBA already contains an MBR then it is OK to use
hexdump -q -s 0x1fe -n 2 (${grubfm_disk})$LBA+1 x
if [ "$x" = "\x55\xaa" ] ; then 
	hexdump -q -s 0x1c2 -n 1 (${grubfm_disk})$LBA+1 x
	if [ "$x" = "\x07" ] ; then 
		hexdump -q -s 0x1d2 -n 1 (${grubfm_disk})$LBA+1 x
		if [ "$x" = "\x0C" -o "$x" = "\x0B" ] ; then 
			hexdump -q -s 0x1be -n 1 (${grubfm_disk})$LBA+1 x
			if [ "$x" = "\x00" -o "$x" = "\x80" ] ; then
				if [ -f (hd0,msdos1)/_ISO/e2b/grub/E2B.cfg ] ; then set USED=0 ; echo LBA$LBA is valid ; fi
			fi
		fi
	fi
fi
unset x
unset LBA
}

source ${prefix}/func.sh;

#check file is contiguous
#stat -q -c -s contig "${grubfm_file}"
#if [ "${contig}" = "1" ] ; then 
#	echo "ERROR: File is not contiguous!"
#	echo "Please run \MAKE_THIS_DRIVE_CONTIGUOUS.cmd or Defraggler under Windows"
#	echo "Press ENTER to Cancel"
#	read
#fi


if [ "${PTYPE}" = "0" ] ; then
set PTYPE=0x00
#types fat, ext2, reiserfs, btrfs, hfs, hfsplus, ntfs, exfat, sun, sunpc, udf, iso9660, ufs1_bw, ufs1, zfs
#bsd, openbsd, netbsd
loopback tst "${grubfm_file}"
probe --set=fs --fs (tst)
#assume 'fat' is FAT32 type 0x0C
if [ "${fs}" = "fat" ] ; then set PTYPE=0x0C ; fi
if [ "${fs}" = "ntfs" ] ; then set PTYPE=0x07 ; fi
if [ "${fs}" = "exfat" ] ; then set PTYPE=0x07 ; fi
if [ "${fs}" = "ext2" ] ; then set PTYPE=0x83 ; fi
if [ "${fs}" = "ext3" ] ; then set PTYPE=0x83 ; fi
if [ "${fs}" = "ext4" ] ; then set PTYPE=0x83 ; fi
if [ "${fs}" = "ext" ] ; then set PTYPE=0x83 ; fi
if [ "${fs}" = "hfs" ] ; then set PTYPE=0xAF ; fi
if [ "${fs}" = "bsd" ] ; then set PTYPE=0xA5 ; fi
if [ "${fs}" = "openbsd" ] ; then set PTYPE=0xA5 ; fi
if [ "${fs}" = "netbsd" ] ; then set PTYPE=0xA5 ; fi
loopback -d tst
fi


#check for '23' in filename
set ext23=0
if regexp '\..*[2][3]' "$grubfm_file" ; then ext23=1 ; fi

#check if sector is MBR with partition table
check_mbr 0
#test lba to see if OK to use as a backup
if [ "$USED" = "0" ] ; then test_lba 30 ; fi
if [ "$USED" = "0" ] ; then 
	dd --if (hd0)0+1 --of (hd0)30+1 -c 1 -b 512
	echo Backup made to LBA30
fi

#check if sector is MBR with partition table
check_mbr 0
#test lba to see if OK to use as a backup (60 used as second backup)
if [ "$USED" = "0" ] ; then test_lba 60 ; fi
if [ "$USED" = "0" ] ; then 
	dd --if (hd0)0+1 --of (hd0)60+1 -c 1 -b 512
	echo Second backup made to LBA60
fi

#Check backup at LBA30 is now OK
check_mbr 30
if [ ! "$USED" = "0" ] ; then echo Problem making backup to LBA30 ; read ; grubfm_open "${g}" ; fi

set g="${grubfm_file}"
if [ "$USED" = "0" ]; then
  echo
  echo "SWITCH .IMGPTN FILE INTO PARTITION 1"
  echo "===================================="
  echo
  echo IMAGE: $grubfm_file
  echo "TYPE:  $PTYPE ${fs}"
  echo
  echo   "Replace Partition 1"
  if [ "$ext23" = "0" ] ; then 
	echo "Delete  Partition 2"
	echo "Delete  Partition 3"
  else
	echo "Keep    Partition 2"
	echo "Keep    Partition 3"  
  fi
  echo
  echo "Press [1] to change partitions (any other key to cancel).";
  getkey key;
else
  key=1;
fi;

if [ x$key = x49 ]; then
	echo
	echo Updating Partition 1...
	partnew -a --type="${PTYPE}" --file="${grubfm_file}" (${grubfm_disk}) 1
	# delete ptns 2 and 3 if extension has 23 in it
	if [ "$ext23" = "0" ] ; then partnew --type=0 --start=0 --length=0 (${grubfm_device}) 2 ; echo Ptn2 deleted ; fi
	if [ "$ext23" = "0" ] ; then partnew --type=0 --start=0 --length=0 (${grubfm_device}) 3 ; echo Ptn3 deleted ; fi
	if [ "$grub_platform" = "efi" ] ; then
		echo
		echo
		echo Partition 1 changed
		echo On reboot to File Manager, press F3 and boot from (hd0,msdos1)
		echo Rebooting...
		sleep 3
		reboot
	elif [ "$grub_platform" = "pc" ] ; then
		chainloader (hd0)+1
		boot
	fi;
else
	echo "Cancelled."
	sleep 2
	grubfm_open "${g}"
fi;

